@@ -1,7 +1,7 @@
 from unittest import TestCase
 from hamcrest import *
 from deftlariat import AnythingMatcher, EqualTo, \
-    Matcher, MatcherType, NumberComparer, TextComparer, ExistsMatchers
+    Matcher, MatcherType, NumberComparer, TextComparer, ExistsMatchers, DictMatchers
 
 
 class TestAnythingMatcher(TestCase):
@@ -77,6 +77,18 @@ class TestEqualTo(TestCase):
         self.assertFalse(name_check.is_match(target_value, test_data_record),
                          "Check against single item list")
 
+    def test_boolean(self):
+        test_data_record = {'full_name': True}
+
+        target_value = [False]
+        name_check = EqualTo('full_name')
+
+        self.assertFalse(name_check.is_match(target_value, test_data_record),
+                         "Check boolean no match")
+
+        self.assertTrue(name_check.is_match([True], test_data_record),
+                        "Check boolean does match")
+
 
 class TestTextCompareBoundaryCases(TestCase):
     def setUp(self) -> None:
@@ -290,6 +302,7 @@ class TestNumberComparisons(TestCase):
         self.assertFalse(self.number_close_check.is_match((target_value, 1), test_data_record),
                          " 5 close to (10,2) = False")
 
+
 class TestNumberComparisonsWithReplacement(TestCase):
 
     def setUp(self):
@@ -333,8 +346,6 @@ class TestNumberComparisonsWithReplacement(TestCase):
         self.assertFalse(self.number_close_check.is_match((target_value, 1), test_data_record),
                          " 5 close to (10,2) = False")
 
-
-
     def test_is_gt_list_value(self):
         test_data_record = {'my_number': 5}
 
@@ -447,18 +458,15 @@ class TestExistenceMatcherBoundaryCases(TestCase):
         for m_type in exists_matcher_types:
             self.my_exists_comparer_list.append(ExistsMatchers('number_col', m_type))
 
-
     def test_key_not_in_data_record(self):
         test_data_record = {'some_other_columns': 11223344}
 
         for comparer in self.my_exists_comparer_list:
             target_value = []
 
-            assert_that(comparer.is_match( test_data_record),
+            assert_that(comparer.is_match(test_data_record),
                         equal_to(False), "Return False when field not available.")
 
-
-
     def test_bad_matcher_type(self):
         with self.assertRaises(NotImplementedError):
             self.comparer = NumberComparer("Foo", MatcherType.STARTS_WITH)
@@ -557,3 +565,91 @@ class TestExistsMatchers(TestCase):
         test_data_record = {'some_col': ()}
         assert_that(my_is_none.is_match(test_data_record), equal_to(False),
                     "It is actually None")
+
+
+class TestDictMatchers(TestCase):
+
+    def test_has_entries(self):
+        operator = DictMatchers('dumb', MatcherType.HAS_ENTRIES)
+        assert_that(operator.is_match('a', 'b', data_record={'dumb': {'a': 'b'}}),
+                    equal_to(True))
+
+        assert_that(operator.is_match({'a': 'b'}, data_record={'dumb': {'a': 'b'}}),
+                    equal_to(True))
+
+        assert_that(operator.is_match({'foo': 'bar'}, data_record={'dumb': {'a': 'b'}}),
+                    equal_to(False))
+        assert_that(operator.is_match('foo', 'bar', data_record={'dumb': {'a': 'b'}}),
+                    equal_to(False))
+
+        assert_that(operator.is_match('a', 'b', 'c', 'd',
+                                      data_record={'dumb': {'c': 'd', 'a': 'b'}}),
+                    equal_to(True))
+
+        assert_that(operator.is_match('a', 'b',
+                                      data_record={'dumb': [
+                                          {'1': '2'}, {'c': 'd'}, {'a': 'b'}]}),
+                    equal_to(True))
+
+        assert_that(operator.is_match([{'a': 'b'}, {'1': '2'}],
+                                      data_record={'dumb': [
+                                          {'1': '2'}, {'c': 'd'}, {'a': 'b'}]}),
+                    equal_to(True))
+
+        assert_that(operator.is_match([{'a': 'g'}, {'1': 'g'}],
+                                      data_record={'dumb': [
+                                          {'1': '2'}, {'c': 'd'}, {'a': 'b'}]}),
+                    equal_to(False))
+
+        assert_that(operator.is_match([{'a': 'g'}, {'1': 'g'}],
+                                      data_record={'dumb': {'a': 'b'}}),
+                    equal_to(False))
+
+        assert_that(operator.is_match([{'a': 'b'}, {'1': '2'}],
+                                      data_record={'dumb': {'a': 'b'}}),
+                    equal_to(True))
+
+    def test_has_entry(self):
+        operator = DictMatchers('dumb', MatcherType.HAS_ENTRY)
+        assert_that(operator.is_match('a', 'b', data_record={'dumb': {'a': 'b'}}),
+                    equal_to(True))
+
+        assert_that(operator.is_match({'a': 'b'}, data_record={'dumb': {'a': 'b'}}),
+                    equal_to(True))
+
+        assert_that(operator.is_match({'foo': 'bar'}, data_record={'dumb': {'a': 'b'}}),
+                    equal_to(False))
+        assert_that(operator.is_match('foo', 'bar', data_record={'dumb': {'a': 'b'}}),
+                    equal_to(False))
+
+        assert_that(operator.is_match('a', 'b',
+                                      data_record={'dumb': [
+                                          {'1': '2'}, {'c': 'd'}, {'a': 'b'}]}),
+                    equal_to(True))
+
+        assert_that(operator.is_match([{'a': 'b'}, {'1': '2'}],
+                                      data_record={'dumb': [
+                                          {'1': '2'}, {'c': 'd'}, {'a': 'b'}]}),
+                    equal_to(True))
+
+        assert_that(operator.is_match([{'a': 'g'}, {'1': 'g'}],
+                                      data_record={'dumb': [
+                                          {'1': '2'}, {'c': 'd'}, {'a': 'b'}]}),
+                    equal_to(False))
+
+        assert_that(operator.is_match([{'a': 'g'}, {'1': 'g'}],
+                                      data_record={'dumb': {'a': 'b'}}),
+                    equal_to(False))
+
+        assert_that(operator.is_match([{'a': 'b'}, {'1': '2'}],
+                                      data_record={'dumb': {'a': 'b'}}),
+                    equal_to(True))
+
+    def test_has_entry_bad_input(self):
+        operator = DictMatchers('dumb', MatcherType.HAS_ENTRY)
+
+        with self.assertRaises(ValueError):
+            assert_that(operator.is_match('a', 'b', 'c', 'd',
+                                          data_record={'dumb': {'c': 'd', 'a': 'b'}}),
+                        equal_to(True))
+
